@@ -9,12 +9,6 @@ class TestDuckDB(Validator):
     dialect = "duckdb"
 
     def test_duckdb(self):
-        self.validate_identity("SELECT UUIDV7()")
-        self.validate_identity("SELECT TRY(LOG(0))")
-        self.validate_identity("x::timestamp", "CAST(x AS TIMESTAMP)")
-        self.validate_identity("x::timestamp without time zone", "CAST(x AS TIMESTAMP)")
-        self.validate_identity("x::timestamp with time zone", "CAST(x AS TIMESTAMPTZ)")
-
         with self.assertRaises(ParseError):
             parse_one("1 //", read="duckdb")
 
@@ -35,6 +29,20 @@ class TestDuckDB(Validator):
             "STRUCT(k TEXT, v STRUCT(v_str TEXT, v_int INT, v_int_arr INT[]))[]",
         )
 
+        self.validate_all(
+            "SELECT FIRST_VALUE(c IGNORE NULLS) OVER (PARTITION BY gb ORDER BY ob) FROM t",
+            write={
+                "duckdb": "SELECT FIRST_VALUE(c IGNORE NULLS) OVER (PARTITION BY gb ORDER BY ob) FROM t",
+                "sqlite": UnsupportedError,
+            },
+        )
+        self.validate_all(
+            "SELECT FIRST_VALUE(c RESPECT NULLS) OVER (PARTITION BY gb ORDER BY ob) FROM t",
+            write={
+                "duckdb": "SELECT FIRST_VALUE(c RESPECT NULLS) OVER (PARTITION BY gb ORDER BY ob) FROM t",
+                "sqlite": "SELECT FIRST_VALUE(c) OVER (PARTITION BY gb ORDER BY ob NULLS LAST) FROM t",
+            },
+        )
         self.validate_all(
             "CAST(x AS UUID)",
             write={
@@ -263,6 +271,12 @@ class TestDuckDB(Validator):
             parse_one("a // b", read="duckdb").assert_is(exp.IntDiv).sql(dialect="duckdb"), "a // b"
         )
 
+        self.validate_identity("SELECT * FROM my_ducklake.demo AT (VERSION => 2)")
+        self.validate_identity("SELECT UUIDV7()")
+        self.validate_identity("SELECT TRY(LOG(0))")
+        self.validate_identity("x::timestamp", "CAST(x AS TIMESTAMP)")
+        self.validate_identity("x::timestamp without time zone", "CAST(x AS TIMESTAMP)")
+        self.validate_identity("x::timestamp with time zone", "CAST(x AS TIMESTAMPTZ)")
         self.validate_identity("CAST(x AS FOO)")
         self.validate_identity("SELECT UNNEST([1, 2])").selects[0].assert_is(exp.UDTF)
         self.validate_identity("'red' IN flags").args["field"].assert_is(exp.Column)
@@ -567,6 +581,9 @@ class TestDuckDB(Validator):
         )
         self.validate_all(
             "STRING_TO_ARRAY(x, 'a')",
+            read={
+                "snowflake": "STRTOK_TO_ARRAY(x, 'a')",
+            },
             write={
                 "duckdb": "STR_SPLIT(x, 'a')",
                 "presto": "SPLIT(x, 'a')",
@@ -946,6 +963,8 @@ class TestDuckDB(Validator):
             "SELECT CAST('2020-01-01' AS DATE) + INTERVAL '1' DAY",
         )
 
+        self.validate_identity("ARRAY_SLICE(x, 1, 3, 2)")
+
     def test_array_index(self):
         with self.assertLogs(helper_logger) as cm:
             self.validate_all(
@@ -1477,7 +1496,11 @@ class TestDuckDB(Validator):
 
         # DETACH
         self.validate_identity("DETACH new_database")
-        self.validate_identity("DETACH IF EXISTS file")
+
+        # when 'if exists' is set, the syntax is DETACH DATABASE, not DETACH
+        # ref: https://duckdb.org/docs/stable/sql/statements/attach.html#detach-syntax
+        self.validate_identity("DETACH IF EXISTS file", "DETACH DATABASE IF EXISTS file")
+        self.validate_identity("DETACH DATABASE IF EXISTS file", "DETACH DATABASE IF EXISTS file")
 
         self.validate_identity("DETACH DATABASE db", "DETACH db")
 
@@ -1635,3 +1658,20 @@ class TestDuckDB(Validator):
     def test_show_tables(self):
         self.validate_identity("SHOW TABLES").assert_is(exp.Show)
         self.validate_identity("SHOW ALL TABLES").assert_is(exp.Show)
+
+    def test_extract_date_parts(self):
+        for part in ("WEEK", "WEEKOFYEAR"):
+            # Both are synonyms for ISO week
+            self.validate_identity(f"EXTRACT({part} FROM foo)", "EXTRACT(WEEK FROM foo)")
+
+        for part in (
+            "WEEKDAY",
+            "ISOYEAR",
+            "ISODOW",
+            "YEARWEEK",
+            "TIMEZONE_HOUR",
+            "TIMEZONE_MINUTE",
+        ):
+            with self.subTest(f"Testing DuckDB EXTRACT({part} FROM foo)"):
+                # All of these should remain as is, they don't have synonyms
+                self.validate_identity(f"EXTRACT({part} FROM foo)")
