@@ -171,7 +171,8 @@ SPLIT('new york', '.')
 X((y AS z)).1
 X(a.b = 1)
 (x AS y, y AS z)
-REPLACE(1)
+REPLACE('new york', ' ', '_')
+REPLACE('new york', ' ')
 DATE(x) = DATE(y)
 TIMESTAMP(DATE(x))
 TIMESTAMP_TRUNC(COALESCE(time_field, CURRENT_TIMESTAMP()), DAY)
@@ -791,6 +792,7 @@ ALTER TABLE pets ADD CONSTRAINT pets_persons_fk FOREIGN KEY (owner_first_name, o
 ALTER TABLE pets ADD CONSTRAINT pets_name_not_cute_chk CHECK (LENGTH(name) < 20)
 ALTER TABLE people10m ADD CONSTRAINT dateWithinRange CHECK (birthDate > '1900-01-01')
 ALTER TABLE people10m ADD CONSTRAINT validIds CHECK (id > 1 AND id < 99999999) ENFORCED
+ALTER TABLE ct ADD CONSTRAINT ct_id_fk FOREIGN KEY (id) REFERENCES et (fid) DEFERRABLE INITIALLY DEFERRED
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo FOREIGN KEY (x, y) REFERENCES persons ON UPDATE NO ACTION ON DELETE NO ACTION MATCH FULL
@@ -830,8 +832,6 @@ JSON_OBJECT('x': 1 RETURNING VARCHAR(100))
 JSON_OBJECT('x': 1 RETURNING VARBINARY FORMAT JSON ENCODING UTF8)
 PRIOR AS x
 SELECT if.x
-SELECT NEXT VALUE FOR db.schema.sequence_name
-SELECT NEXT VALUE FOR db.schema.sequence_name OVER (ORDER BY foo), col
 SELECT PERCENTILE_CONT(x, 0.5) OVER ()
 WITH my_cte AS (SELECT 'a' AS desc) SELECT desc AS description FROM my_cte
 WITH my_cte AS (SELECT 'a' AS asc) SELECT asc AS description FROM my_cte
@@ -882,4 +882,16 @@ GRANT SELECT ON TABLE tbl TO user
 GRANT SELECT, INSERT ON FUNCTION tbl TO user
 GRANT SELECT ON orders TO ROLE PUBLIC
 GRANT SELECT ON nation TO alice WITH GRANT OPTION
-GRANT DELETE ON SCHEMA finance TO bob
\ No newline at end of file
+GRANT DELETE ON SCHEMA finance TO bob
+SELECT attach
+SELECT detach
+SELECT 1 OFFSET 1
+SELECT 1 LIMIT 1
+CAST(x AS INT128)
+CAST(x AS UINT128)
+CAST(x AS UINT256)
+SELECT export
+SELECT ARG_MAX(DISTINCT selected_col, filtered_col) FROM table
+SELECT ARG_MIN(DISTINCT selected_col, filtered_col) FROM table
+a.b.c.D()
+a.b.c.d.e.f.G()
