@@ -14,6 +14,12 @@ class TestParser(unittest.TestCase):
             parse_one("")
 
     def test_parse_into(self):
+        self.assertIsInstance(parse_one("(1)", into=exp.Tuple), exp.Tuple)
+        self.assertIsInstance(parse_one("(1,)", into=exp.Tuple), exp.Tuple)
+        self.assertIsInstance(parse_one("(x=1)", into=exp.Tuple), exp.Tuple)
+
+        self.assertIsInstance(parse_one("select * from t", into=exp.Select), exp.Select)
+        self.assertIsInstance(parse_one("select * from t limit 5", into=exp.Select), exp.Select)
         self.assertIsInstance(parse_one("left join foo", into=exp.Join), exp.Join)
         self.assertIsInstance(parse_one("int", into=exp.DataType), exp.DataType)
         self.assertIsInstance(parse_one("array<int>", into=exp.DataType), exp.DataType)
@@ -21,9 +27,9 @@ class TestParser(unittest.TestCase):
         self.assertIsInstance(
             parse_one(
                 "WHEN MATCHED THEN UPDATE SET target.salary = COALESCE(source.salary, target.salary)",
-                into=exp.When,
+                into=exp.Whens,
             ),
-            exp.When,
+            exp.Whens,
         )
 
         with self.assertRaises(ParseError) as ctx:
@@ -327,7 +333,7 @@ class TestParser(unittest.TestCase):
                 e, --
                 f -- space
             FROM foo
-        """
+            """
         )
 
         self.assertEqual(expression.comments, ["comment1.1", "comment1.2", "comment1.3"])
@@ -338,6 +344,20 @@ class TestParser(unittest.TestCase):
         self.assertEqual(expression.expressions[4].comments, [""])
         self.assertEqual(expression.expressions[5].comments, [" space"])
 
+        expression = parse_one(
+            """
+            SELECT a.column_name --# Comment 1
+                   ,b.column_name2, --# Comment 2
+                   b.column_name3 AS NAME3 --# Comment 3
+            FROM table_name a
+            JOIN table_name2 b ON a.column_name = b.column_name
+            """
+        )
+
+        self.assertEqual(expression.expressions[0].comments, ["# Comment 1"])
+        self.assertEqual(expression.expressions[1].comments, ["# Comment 2"])
+        self.assertEqual(expression.expressions[2].comments, ["# Comment 3"])
+
     def test_comments_select_cte(self):
         expression = parse_one(
             """
@@ -348,7 +368,7 @@ class TestParser(unittest.TestCase):
                 a.*
             FROM /*comment3*/
                 a
-        """
+            """
         )
 
         self.assertEqual(expression.comments, ["comment2"])
@@ -577,12 +597,6 @@ class TestParser(unittest.TestCase):
             logger,
         )
 
-    def test_rename_table(self):
-        self.assertEqual(
-            parse_one("ALTER TABLE foo RENAME TO bar").sql(),
-            "ALTER TABLE foo RENAME TO bar",
-        )
-
     def test_pivot_columns(self):
         nothing_aliased = """
             SELECT * FROM (
@@ -632,6 +646,27 @@ class TestParser(unittest.TestCase):
             ) PIVOT (AVG("PrIcE"), MAX(quality) FOR partname IN ('prop' AS prop1, 'rudder'))
         """
 
+        two_in_clauses_duckdb = """
+            SELECT * FROM cities PIVOT (
+                sum(population) AS total,
+                count(population) AS count
+                FOR
+                    year IN (2000, 2010)
+                    country IN ('NL', 'US')
+            )
+        """
+
+        three_in_clauses_duckdb = """
+            SELECT * FROM cities PIVOT (
+                sum(population) AS total,
+                count(population) AS count
+                FOR
+                    year IN (2000, 2010)
+                    country IN ('NL', 'US')
+                    name IN ('Amsterdam', 'Seattle')
+            )
+        """
+
         query_to_column_names = {
             nothing_aliased: {
                 "bigquery": ["prop", "rudder"],
@@ -693,87 +728,64 @@ class TestParser(unittest.TestCase):
                     '"rudder_max(quality)"',
                 ],
             },
+            two_in_clauses_duckdb: {
+                "duckdb": [
+                    '"2000_NL_total"',
+                    '"2000_NL_count"',
+                    '"2000_US_total"',
+                    '"2000_US_count"',
+                    '"2010_NL_total"',
+                    '"2010_NL_count"',
+                    '"2010_US_total"',
+                    '"2010_US_count"',
+                ],
+            },
+            three_in_clauses_duckdb: {
+                "duckdb": [
+                    '"2000_NL_Amsterdam_total"',
+                    '"2000_NL_Amsterdam_count"',
+                    '"2000_NL_Seattle_total"',
+                    '"2000_NL_Seattle_count"',
+                    '"2000_US_Amsterdam_total"',
+                    '"2000_US_Amsterdam_count"',
+                    '"2000_US_Seattle_total"',
+                    '"2000_US_Seattle_count"',
+                    '"2010_NL_Amsterdam_total"',
+                    '"2010_NL_Amsterdam_count"',
+                    '"2010_NL_Seattle_total"',
+                    '"2010_NL_Seattle_count"',
+                    '"2010_US_Amsterdam_total"',
+                    '"2010_US_Amsterdam_count"',
+                    '"2010_US_Seattle_total"',
+                    '"2010_US_Seattle_count"',
+                ],
+            },
         }
 
         for query, dialect_columns in query_to_column_names.items():
             for dialect, expected_columns in dialect_columns.items():
-                expr = parse_one(query, read=dialect)
-                columns = expr.args["from"].this.args["pivots"][0].args["columns"]
-                self.assertEqual(expected_columns, [col.sql(dialect=dialect) for col in columns])
+                with self.subTest(f"Testing query '{query}' for dialect {dialect}"):
+                    expr = parse_one(query, read=dialect)
+                    columns = expr.args["from"].this.args["pivots"][0].args["columns"]
+                    self.assertEqual(
+                        expected_columns, [col.sql(dialect=dialect) for col in columns]
+                    )
 
     def test_parse_nested(self):
-        now = time.time()
-        query = parse_one(
-            """
-            SELECT *
-            FROM a
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            LEFT JOIN b ON a.id = b.id
-            """
-        )
+        def warn_over_threshold(query: str, max_threshold: float = 0.2):
+            now = time.time()
+            ast = parse_one(query)
+            end = time.time() - now
 
-        self.assertIsNotNone(query)
+            self.assertIsNotNone(ast)
+            if end >= max_threshold:
+                parser_logger.warning(
+                    f"Query {query[:100]}... surpassed the time threshold of {max_threshold} seconds"
+                )
 
-        query = parse_one(
-            """
-            SELECT *
-            FROM a
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            LEFT JOIN UNNEST(ARRAY[])
-            """
-        )
-
-        self.assertIsNotNone(query)
-        self.assertLessEqual(time.time() - now, 0.2)
+        warn_over_threshold("SELECT * FROM a " + ("LEFT JOIN b ON a.id = b.id " * 38))
+        warn_over_threshold("SELECT * FROM a " + ("LEFT JOIN UNNEST(ARRAY[]) " * 15))
+        warn_over_threshold("SELECT * FROM a " + ("OUTER APPLY (SELECT * FROM b) " * 30))
 
     def test_parse_properties(self):
         self.assertEqual(
@@ -901,3 +913,99 @@ class TestParser(unittest.TestCase):
 
     def test_parse_prop_eq(self):
         self.assertIsInstance(parse_one("x(a := b and c)").expressions[0], exp.PropertyEQ)
+
+    def test_collate(self):
+        collates = [
+            ('pg_catalog."default"', exp.Column),
+            ('"en_DE"', exp.Identifier),
+            ("LATIN1_GENERAL_BIN", exp.Var),
+            ("'en'", exp.Literal),
+        ]
+
+        for collate_pair in collates:
+            collate_node = parse_one(
+                f"""SELECT * FROM t WHERE foo LIKE '%bar%' COLLATE {collate_pair[0]}"""
+            ).find(exp.Collate)
+            self.assertIsInstance(collate_node, exp.Collate)
+            self.assertIsInstance(collate_node.expression, collate_pair[1])
+
+    def test_odbc_date_literals(self):
+        for value, cls in [
+            ("{d'2024-01-01'}", exp.Date),
+            ("{t'12:00:00'}", exp.Time),
+            ("{ts'2024-01-01 12:00:00'}", exp.Timestamp),
+        ]:
+            sql = f"INSERT INTO tab(ds) VALUES ({value})"
+            expr = parse_one(sql)
+            self.assertIsInstance(expr, exp.Insert)
+            self.assertIsInstance(expr.expression.expressions[0].expressions[0], cls)
+
+    def test_drop_column(self):
+        ast = parse_one("ALTER TABLE tbl DROP COLUMN col")
+        self.assertEqual(len(list(ast.find_all(exp.Table))), 1)
+        self.assertEqual(len(list(ast.find_all(exp.Column))), 1)
+
+    def test_udf_meta(self):
+        ast = parse_one("YEAR(a) /* sqlglot.anonymous */")
+        self.assertIsInstance(ast, exp.Anonymous)
+
+        # Meta flag is case sensitive
+        ast = parse_one("YEAR(a) /* sqlglot.anONymous */")
+        self.assertIsInstance(ast, exp.Year)
+
+        # Incomplete or incorrect anonymous meta comments are not registered
+        ast = parse_one("YEAR(a) /* sqlglot.anon */")
+        self.assertIsInstance(ast, exp.Year)
+
+    def test_token_position_meta(self):
+        ast = parse_one(
+            "SELECT a, b FROM test_schema.test_table_a UNION ALL SELECT c, d FROM test_catalog.test_schema.test_table_b"
+        )
+        for identifier in ast.find_all(exp.Identifier):
+            self.assertEqual(set(identifier.meta), {"line", "col", "start", "end"})
+
+        self.assertEqual(
+            ast.this.args["from"].this.args["this"].meta,
+            {"line": 1, "col": 41, "start": 29, "end": 40},
+        )
+        self.assertEqual(
+            ast.this.args["from"].this.args["db"].meta,
+            {"line": 1, "col": 28, "start": 17, "end": 27},
+        )
+        self.assertEqual(
+            ast.expression.args["from"].this.args["this"].meta,
+            {"line": 1, "col": 106, "start": 94, "end": 105},
+        )
+        self.assertEqual(
+            ast.expression.args["from"].this.args["db"].meta,
+            {"line": 1, "col": 93, "start": 82, "end": 92},
+        )
+        self.assertEqual(
+            ast.expression.args["from"].this.args["catalog"].meta,
+            {"line": 1, "col": 81, "start": 69, "end": 80},
+        )
+
+        ast = parse_one("SELECT FOO()")
+        self.assertEqual(ast.find(exp.Anonymous).meta, {"line": 1, "col": 10, "start": 7, "end": 9})
+
+        ast = parse_one("SELECT * FROM t")
+        self.assertEqual(ast.find(exp.Star).meta, {"line": 1, "col": 8, "start": 7, "end": 7})
+
+        ast = parse_one("SELECT t.* FROM t")
+        self.assertEqual(ast.find(exp.Star).meta, {"line": 1, "col": 10, "start": 9, "end": 9})
+
+    def test_quoted_identifier_meta(self):
+        sql = 'SELECT "a" FROM "test_schema"."test_table_a"'
+        ast = parse_one(sql)
+
+        db_meta = ast.args["from"].this.args["db"].meta
+        self.assertEqual(sql[db_meta["start"] : db_meta["end"] + 1], '"test_schema"')
+
+        table_meta = ast.args["from"].this.this.meta
+        self.assertEqual(sql[table_meta["start"] : table_meta["end"] + 1], '"test_table_a"')
+
+    def test_qualified_function(self):
+        sql = "a.b.c.d.e.f.g.foo()"
+        ast = parse_one(sql)
+        assert not any(isinstance(n, exp.Column) for n in ast.walk())
+        assert len(list(ast.find_all(exp.Dot))) == 7
